@@ -2,15 +2,22 @@ from __future__ import annotations
 
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
-from typing import Generic, TypeVar
+from typing import TYPE_CHECKING, Any, Generic, TypeVar
+
+from typing_extensions import TypeAlias
 
 from daft.datatype import DataType
-from daft.dependencies import np
 
-Options = dict[str, str]
+Options = dict[str, Any]
 
 T = TypeVar("T")
 
+__all__ = [
+    "Descriptor",
+    "Embedding",
+    "EmbeddingDimensions",
+]
+
 
 class Descriptor(ABC, Generic[T]):
     """A descriptor is a serializable factory, with protocol-specific properties, used to instantiate a model."""
@@ -36,7 +43,12 @@ class Descriptor(ABC, Generic[T]):
 
 
 # temp definition to defer complexity of a more generic embedding type to later PRs
-Embedding = np.typing.NDArray  # type: ignore[type-arg]
+if TYPE_CHECKING:
+    from daft.dependencies import np
+
+    Embedding: TypeAlias = np.typing.NDArray[Any]
+else:
+    Embedding: TypeAlias = Any
 
 
 @dataclass
