@@ -1,19 +1,25 @@
 from __future__ import annotations
+from typing import TYPE_CHECKING
 
-from .window import (
-    row_number,
-    rank,
-    dense_rank,
-)
-from .misc import monotonically_increasing_id, format
-from .columnar import (
+if TYPE_CHECKING:
+    from daft.expressions import Expression
+
+from daft.functions.ai import embed_text
+from daft.functions.columnar import (
     columns_sum,
     columns_mean,
     columns_avg,
     columns_min,
     columns_max,
 )
-from .llm import llm_generate
+from daft.functions.llm import llm_generate
+from daft.functions.misc import monotonically_increasing_id, format, file
+from daft.functions.window import (
+    row_number,
+    rank,
+    dense_rank,
+)
+
 
 __all__ = [
     "columns_avg",
@@ -22,6 +28,8 @@ __all__ = [
     "columns_min",
     "columns_sum",
     "dense_rank",
+    "embed_text",
+    "file",
     "format",
     "llm_generate",
     "monotonically_increasing_id",
