@@ -8,21 +8,24 @@ from daft.daft import (
     S3Config,
     S3Credentials,
     UnityConfig,
+    HuggingFaceConfig,
 )
 from daft.io._csv import read_csv
 from daft.io.delta_lake._deltalake import read_deltalake
 from daft.io.hudi._hudi import read_hudi
 from daft.io.iceberg._iceberg import read_iceberg
-from daft.io.lance._lance import read_lance
+from daft.io.lance._lance import read_lance, merge_columns
 from daft.io._json import read_json
 from daft.io._parquet import read_parquet
 from daft.io._sql import read_sql
 from daft.io._warc import read_warc
+from daft.io.huggingface import read_huggingface
 from daft.io._range import _range
 from daft.io.catalog import DataCatalogTable, DataCatalogType
 from daft.io.file_path import from_glob_path
 from daft.io.sink import DataSink
 from daft.io.source import DataSource, DataSourceTask
+from daft.io.av import read_video_frames
 
 __all__ = [
     "AzureConfig",
@@ -33,19 +36,23 @@ __all__ = [
     "DataSourceTask",
     "GCSConfig",
     "HTTPConfig",
+    "HuggingFaceConfig",
     "IOConfig",
     "S3Config",
     "S3Credentials",
     "UnityConfig",
     "_range",
     "from_glob_path",
+    "merge_columns",
     "read_csv",
     "read_deltalake",
     "read_hudi",
+    "read_huggingface",
     "read_iceberg",
     "read_json",
     "read_lance",
     "read_parquet",
     "read_sql",
+    "read_video_frames",
     "read_warc",
 ]
