@@ -9,7 +9,7 @@ use super::as_arrow::AsArrow;
 use crate::datatypes::PythonArray;
 use crate::{
     array::{DataArray, FixedSizeListArray, ListArray, StructArray},
-    datatypes::DaftArrowBackedType,
+    datatypes::{DaftArrowBackedType, FileArray},
 };
 
 impl<T> DataArray<T>
@@ -109,3 +109,9 @@ impl StructArray {
         children_size_bytes + validity_size(self.validity())
     }
 }
+
+impl FileArray {
+    pub fn size_bytes(&self) -> usize {
+        self.physical.size_bytes()
+    }
+}
