@@ -1,8 +1,7 @@
 mod conversions;
 mod deserializer;
 #[cfg(feature = "python")]
-mod python;
-
+pub(crate) mod python;
 use std::{
     fmt::{Display, Formatter, Result},
     hash::{Hash, Hasher},
@@ -11,6 +10,7 @@ use std::{
 
 use common_display::table_display::StrValue;
 use common_error::{ensure, DaftError, DaftResult};
+use common_file::DaftFile;
 use common_hashable_float_wrapper::FloatWrapper;
 use common_image::{CowImage, Image};
 #[cfg(feature = "python")]
@@ -92,6 +92,7 @@ pub enum Literal {
     Python(PyObjectWrapper),
     /// TODO chore: audit struct literal vs. struct expression support.
     Struct(IndexMap<Field, Literal>),
+    File(DaftFile),
     /// A tensor
     Tensor {
         data: Series,
@@ -100,7 +101,7 @@ pub enum Literal {
     /// A sparse tensor (values, indices, shape, indices_offset)
     SparseTensor {
         values: Series,
-        indices: Vec<u64>,
+        indices: Series,
         shape: Vec<u64>,
         indices_offset: bool,
     },
@@ -113,6 +114,8 @@ pub enum Literal {
     },
     // An image buffer
     Image(Image),
+    // Extension type, stored as a single-element series
+    Extension(Series),
 }
 
 impl Eq for Literal {}
@@ -167,6 +170,7 @@ impl Hash for Literal {
                     f.hash(state);
                 });
             }
+            Self::File(file) => file.hash(state),
             Self::Tensor { data, shape } => {
                 Hash::hash(data, state);
                 shape.hash(state);
@@ -178,7 +182,7 @@ impl Hash for Literal {
                 indices_offset,
             } => {
                 Hash::hash(values, state);
-                indices.hash(state);
+                Hash::hash(indices, state);
                 shape.hash(state);
                 indices_offset.hash(state);
             }
@@ -192,6 +196,7 @@ impl Hash for Literal {
             Self::Image(image_buffer_wrapper) => {
                 image_buffer_wrapper.hash(state);
             }
+            Self::Extension(series) => Hash::hash(series, state),
         }
     }
 }
@@ -239,6 +244,7 @@ impl Display for Literal {
                 }
                 write!(f, ")")
             }
+            Self::File(file) => write!(f, "{file}"),
             Self::Tensor { data, shape } => {
                 write!(
                     f,
@@ -281,6 +287,7 @@ impl Display for Literal {
                 )
             }
             Self::Image(image_buffer_wrapper) => write!(f, "Image({image_buffer_wrapper:?})"),
+            Self::Extension(series) => write!(f, "Extension(\n{}\n)", series),
         }
     }
 }
@@ -317,6 +324,7 @@ impl Literal {
             #[cfg(feature = "python")]
             Self::Python(_) => DataType::Python,
             Self::Struct(entries) => DataType::Struct(entries.keys().cloned().collect()),
+            Self::File(_) => DataType::File,
             Self::Tensor { data, .. } => DataType::Tensor(Box::new(data.data_type().clone())),
             Self::SparseTensor {
                 values,
@@ -333,6 +341,7 @@ impl Literal {
             Self::Image(image_buffer_wrapper) => {
                 DataType::Image(Some(CowImage::from(&image_buffer_wrapper.0).mode()))
             }
+            Self::Extension(series) => series.data_type().clone(),
         }
     }
 
@@ -422,9 +431,11 @@ impl Literal {
             | Self::SparseTensor { .. }
             | Self::Embedding { .. }
             | Self::Map { .. }
-            | Self::Image(_) => display_sql_err,
+            | Self::Image(_)
+            | Self::Extension(_) => display_sql_err,
             #[cfg(feature = "python")]
             Self::Python(..) => display_sql_err,
+            Self::File(_) => display_sql_err,
         }
     }
 
