@@ -6,7 +6,7 @@ use daft_local_plan::LocalPhysicalPlan;
 use daft_logical_plan::stats::StatsState;
 use daft_schema::schema::SchemaRef;
 
-use super::{DistributedPipelineNode, RunningPipelineNode};
+use super::{DistributedPipelineNode, SubmittableTaskStream};
 use crate::{
     pipeline_node::{NodeID, NodeName, PipelineNodeConfig, PipelineNodeContext},
     stage::{StageConfig, StageExecutionContext},
@@ -98,16 +98,15 @@ impl DistributedPipelineNode for FilterNode {
         vec![self.child.clone()]
     }
 
-    fn start(self: Arc<Self>, stage_context: &mut StageExecutionContext) -> RunningPipelineNode {
-        let input_node = self.child.clone().start(stage_context);
+    fn produce_tasks(
+        self: Arc<Self>,
+        stage_context: &mut StageExecutionContext,
+    ) -> SubmittableTaskStream {
+        let input_node = self.child.clone().produce_tasks(stage_context);
 
         let predicate = self.predicate.clone();
-        input_node.pipeline_instruction(stage_context, self, move |input| {
-            Ok(LocalPhysicalPlan::filter(
-                input,
-                predicate.clone(),
-                StatsState::NotMaterialized,
-            ))
+        input_node.pipeline_instruction(self.clone(), move |input| {
+            LocalPhysicalPlan::filter(input, predicate.clone(), StatsState::NotMaterialized)
         })
     }
 
