@@ -4,7 +4,7 @@ use daft_core::{
     series::Series,
 };
 use daft_dsl::{
-    functions::{FunctionArgs, ScalarFunction, ScalarUDF},
+    functions::{scalar::ScalarFn, FunctionArgs, ScalarUDF},
     ExprRef,
 };
 use serde::{Deserialize, Serialize};
@@ -61,11 +61,11 @@ impl ScalarUDF for ListSlice {
             );
         }
 
-        input.to_exploded_field()?.to_list_field()
+        Ok(input.to_exploded_field()?.to_list_field())
     }
 }
 
 #[must_use]
 pub fn list_slice(expr: ExprRef, start: ExprRef, end: ExprRef) -> ExprRef {
-    ScalarFunction::new(ListSlice {}, vec![expr, start, end]).into()
+    ScalarFn::builtin(ListSlice {}, vec![expr, start, end]).into()
 }
