@@ -1,8 +1,6 @@
 mod counting_writer;
-mod image_buffer;
 mod iters;
 pub mod ops;
 use counting_writer::CountingWriter;
-use image_buffer::DaftImageBuffer;
 pub mod functions;
 pub mod series;
