@@ -235,10 +235,24 @@ pub trait ObjectSource: Sync + Send {
 
             let mut continuation_token = lsr.continuation_token.clone();
             while continuation_token.is_some() {
-                let lsr = self.ls(&uri, posix, continuation_token.as_deref(), page_size, io_stats.clone()).await?;
-                continuation_token.clone_from(&lsr.continuation_token);
-                for fm in lsr.files {
-                    yield Ok(fm);
+                // Note: There might some race conditions here that the list response is empty
+                // even though the continuation token of previous response is not empty, so skip NotFound error here.
+                // TODO(desmond): Ideally we should patch how `ls` produces NotFound errors. See issue #4982
+                let lsr_result = self.ls(&uri, posix, continuation_token.as_deref(), page_size, io_stats.clone()).await;
+                match lsr_result {
+                    Ok(lsr) => {
+                        continuation_token.clone_from(&lsr.continuation_token);
+                        for fm in lsr.files {
+                            yield Ok(fm);
+                        }
+                    },
+                    Err(err) => {
+                        if matches!(err, super::Error::NotFound { .. }) {
+                            continuation_token = None;
+                        } else {
+                            yield Err(err);
+                        }
+                    }
                 }
             }
         };
