@@ -1,8 +1,6 @@
-use std::sync::Arc;
-
 use super::ProtoResult;
 use crate::{
-    from_proto_err, non_null, not_implemented_err,
+    non_null, not_implemented_err,
     proto::{from_proto, from_proto_vec, to_proto_vec, ToFromProto},
 };
 
@@ -15,28 +13,39 @@ mod ir {
 #[rustfmt::skip]
 mod proto {
     pub use daft_proto::protos::daft::v1::*;
-    pub use daft_proto::protos::daft::v1::function::Descriptor as FunctionDescriptor;
+    pub use daft_proto::protos::daft::v1::scalar_fn::Descriptor as ScalarFnDescriptor;
 }
 
-/// Handles switching on the descriptor which is hidden behind the expr type.
+/// Handles switching on the variant which is hidden behind the expr type.
 ///
-/// We have special forms in the old FunctionExpr, as well as the PythonUDF which
+/// We have special forms in the old FunctionExpr, as well as the LegacyPythonUDF which
 /// requires special modeling. The common supertype is ir::Expr, so this method
 /// will handle the various representations to simply return an expression.
 /// It's the fact that we have scalar functions nested within two different types
 /// that requires special handling when converting back into the IR i.e. we have
 /// to figure out which type we are splitting into.
-pub fn from_proto_function(message: proto::Function) -> ProtoResult<ir::Expr> {
+pub fn from_proto_function(message: proto::ScalarFn) -> ProtoResult<ir::Expr> {
     let args = ir::functions::FunctionArgs::from_proto(non_null!(message.args))?;
     let expr = match non_null!(message.descriptor) {
-        proto::FunctionDescriptor::Py(py) => {
-            // handle python functions
-            let func = ir::functions::python::PythonUDF::from_proto(py)?;
-            let args = args.into_inner();
-            ir::rex::from_py_func(func, args)
-        }
-        proto::FunctionDescriptor::Rs(rs) => {
-            // handle special form, otherwise it's a ScalarFunction
+        proto::scalar_fn::Descriptor::Py(py) => match non_null!(py.variant) {
+            proto::scalar_fn::py_fn::Variant::Legacy(legacy_fn) => {
+                let func = ir::functions::python::LegacyPythonUDF::from_proto(legacy_fn)?;
+                let args = args.into_inner();
+                ir::rex::from_py_legacy_func(func, args)
+            }
+            proto::scalar_fn::py_fn::Variant::RowWise(row_wise_fn) => {
+                let func = ir::functions::RowWisePyFn {
+                    function_name: row_wise_fn.name.into(),
+                    inner: from_proto(row_wise_fn.inner)?,
+                    return_dtype: from_proto(row_wise_fn.return_dtype)?,
+                    original_args: from_proto(row_wise_fn.original_args)?,
+                    args: args.into_inner(),
+                };
+                ir::rex::from_py_rowwise_func(func)
+            }
+        },
+        proto::scalar_fn::Descriptor::Builtin(builtin_fn) => {
+            // handle special form, otherwise it's a ScalarFn
             // match from_special_form(&rs)? {
             //     Some(_) => {
             //         //
@@ -50,16 +59,58 @@ pub fn from_proto_function(message: proto::Function) -> ProtoResult<ir::Expr> {
             // signatures via string mangling or other techniques. For now, it suffices to lookup
             // the dynamic functions by name because all functions are dynamic. This
             let schema = ir::Schema::empty();
-            let func = ir::functions::get_function(&rs.name); // resolve logical function from name
+            let func = ir::functions::get_function(&builtin_fn.name); // resolve logical function from name
             let func = func.get_function(args.clone(), &schema)?; // resolve physical function from types (todo)
-            ir::rex::from_rs_func(func, args)
+            ir::rex::from_builtin_func(func, args)
         }
     };
     Ok(expr)
 }
 
+pub fn scalar_fn_to_proto(sf: &ir::functions::scalar::ScalarFn) -> ProtoResult<proto::ScalarFn> {
+    match sf {
+        ir::functions::scalar::ScalarFn::Builtin(builtin_scalar_fn) => {
+            let args = builtin_scalar_fn.inputs.to_proto()?;
+            Ok(proto::ScalarFn {
+                descriptor: Some(proto::scalar_fn::Descriptor::Builtin(
+                    proto::scalar_fn::BuiltinFn {
+                        name: builtin_scalar_fn.name().to_string(),
+                    },
+                )),
+                args: Some(args),
+            })
+        }
+        ir::functions::scalar::ScalarFn::Python(ir::functions::PyScalarFn::RowWise(
+            row_wise_fn,
+        )) => {
+            // Convert all arguments to unbound arguments (aka no param name) then reuse existing conversion logic.
+            let function_args = row_wise_fn
+                .args
+                .iter()
+                .map(|arg| ir::functions::FunctionArg::Unnamed(arg.clone()))
+                .collect();
+            let function_args = ir::functions::FunctionArgs::new_unchecked(function_args);
+            let args = function_args.to_proto()?;
+
+            Ok(proto::ScalarFn {
+                descriptor: Some(proto::scalar_fn::Descriptor::Py(proto::scalar_fn::PyFn {
+                    variant: Some(proto::scalar_fn::py_fn::Variant::RowWise(
+                        proto::scalar_fn::py_fn::RowWiseFn {
+                            name: row_wise_fn.function_name.to_string(),
+                            return_dtype: Some(row_wise_fn.return_dtype.to_proto()?),
+                            inner: Some(row_wise_fn.inner.to_proto()?),
+                            original_args: Some(row_wise_fn.original_args.to_proto()?),
+                        },
+                    )),
+                })),
+                args: Some(args),
+            })
+        }
+    }
+}
+
 /// The Expr::Function holds its args, so we can impl ToFromProto on it.
-/// Also, the proto::FunctionDescriptor is an enum, not a message, so we also can't
+/// Also, the proto::ScalarFnDescriptor is an enum, not a message, so we also can't
 /// implement ToFromProto on the FunctionExpr to Descriptor. FunctionExpr is also
 /// either an RsDescriptor or PyDescriptor. All this together means the simplest method
 /// is a custom to_proto implementation. Finally FunctionExpr doesn't use FunctionArgs
@@ -68,30 +119,32 @@ pub fn from_proto_function(message: proto::Function) -> ProtoResult<ir::Expr> {
 pub fn function_expr_to_proto(
     func: &ir::functions::FunctionExpr,
     args: &[ir::ExprRef],
-) -> ProtoResult<proto::Function> {
+) -> ProtoResult<proto::ScalarFn> {
     // build the args
 
     // switch
     let descriptor = match func {
         ir::functions::FunctionExpr::Map(map_expr) => {
             let rs = map_expr.to_proto()?;
-            proto::FunctionDescriptor::Rs(rs)
+            proto::ScalarFnDescriptor::Builtin(rs)
         }
         ir::functions::FunctionExpr::Sketch(sketch_expr) => {
             let rs = sketch_expr.to_proto()?;
-            proto::FunctionDescriptor::Rs(rs)
+            proto::ScalarFnDescriptor::Builtin(rs)
         }
         ir::functions::FunctionExpr::Struct(struct_expr) => {
             let rs = struct_expr.to_proto()?;
-            proto::FunctionDescriptor::Rs(rs)
+            proto::ScalarFnDescriptor::Builtin(rs)
         }
         ir::functions::FunctionExpr::Partitioning(partitioning_expr) => {
             let rs = partitioning_expr.to_proto()?;
-            proto::FunctionDescriptor::Rs(rs)
+            proto::ScalarFnDescriptor::Builtin(rs)
         }
         ir::functions::FunctionExpr::Python(python_udf) => {
             let py = python_udf.to_proto()?;
-            proto::FunctionDescriptor::Py(py)
+            proto::ScalarFnDescriptor::Py(proto::scalar_fn::PyFn {
+                variant: Some(proto::scalar_fn::py_fn::Variant::Legacy(py)),
+            })
         }
     };
 
@@ -103,56 +156,15 @@ pub fn function_expr_to_proto(
     let function_args = ir::functions::FunctionArgs::new_unchecked(function_args);
     let args = function_args.to_proto()?;
 
-    Ok(proto::Function {
+    Ok(proto::ScalarFn {
         descriptor: Some(descriptor),
         args: Some(args),
     })
 }
 
-/// This is to turn a proto rs function descriptor into the rust factory for linking.
-impl ToFromProto for ir::functions::ScalarFunction {
-    type Message = proto::Function;
-
-    fn from_proto(_: Self::Message) -> ProtoResult<Self>
-    where
-        Self: Sized,
-    {
-        from_proto_err!(
-            "ScalarFunction::from_proto called directly, please use `from_proto_function` instead."
-        )
-    }
-
-    fn to_proto(&self) -> ProtoResult<Self::Message> {
-        let args = self.inputs.to_proto()?;
-        let descriptor = proto::FunctionDescriptor::Rs(proto::function::RsFunction {
-            name: self.name().to_string(),
-        });
-        Ok(Self::Message {
-            descriptor: Some(descriptor),
-            args: Some(args),
-        })
-    }
-}
-
-impl ToFromProto for Arc<dyn ir::functions::ScalarFunctionFactory + 'static> {
-    type Message = proto::function::RsFunction;
-
-    fn from_proto(message: Self::Message) -> ProtoResult<Self> {
-        // Lookup in registry by name (that's all we have atm).
-        let name = message.name;
-        let func = ir::functions::get_function(&name);
-        Ok(func)
-    }
-
-    fn to_proto(&self) -> ProtoResult<Self::Message> {
-        let name = self.name().to_string();
-        Ok(Self::Message { name })
-    }
-}
-
 /// FunctionArgs are not bound but are a representation of *how* the customer passed arguments.
 impl ToFromProto for ir::functions::FunctionArgs<ir::ExprRef> {
-    type Message = proto::function::Args;
+    type Message = proto::scalar_fn::Args;
 
     fn from_proto(message: Self::Message) -> ProtoResult<Self>
     where
@@ -164,13 +176,13 @@ impl ToFromProto for ir::functions::FunctionArgs<ir::ExprRef> {
 
     fn to_proto(&self) -> ProtoResult<Self::Message> {
         let args = to_proto_vec(self.iter())?;
-        Ok(proto::function::Args { args })
+        Ok(Self::Message { args })
     }
 }
 
 /// FunctionArg was passed either named or not, in the future we want ALL arguments to be bound to their parameter.
 impl ToFromProto for ir::functions::FunctionArg<ir::ExprRef> {
-    type Message = proto::function::Arg;
+    type Message = proto::scalar_fn::Arg;
 
     fn from_proto(message: Self::Message) -> ProtoResult<Self>
     where
@@ -187,11 +199,11 @@ impl ToFromProto for ir::functions::FunctionArg<ir::ExprRef> {
 
     fn to_proto(&self) -> ProtoResult<Self::Message> {
         let arg = match self {
-            Self::Named { name, arg } => proto::function::Arg {
+            Self::Named { name, arg } => Self::Message {
                 param: name.to_string().into(),
                 expr: arg.to_proto()?.into(),
             },
-            Self::Unnamed(arg) => proto::function::Arg {
+            Self::Unnamed(arg) => Self::Message {
                 param: None,
                 expr: arg.to_proto()?.into(),
             },
@@ -214,7 +226,7 @@ impl ToFromProto for ir::functions::FunctionArg<ir::ExprRef> {
 /// quite simple to go in/out at the expense of some hackery.
 #[allow(unused)]
 fn from_special_form(
-    message: proto::function::RsFunction,
+    message: proto::scalar_fn::BuiltinFn,
 ) -> ProtoResult<Option<ir::functions::FunctionExpr>> {
     let sf = match message.name.as_str() {
         "_map_get" => {
@@ -266,7 +278,7 @@ fn from_special_form(
 }
 
 impl ToFromProto for ir::functions::map::MapExpr {
-    type Message = proto::function::RsFunction;
+    type Message = proto::scalar_fn::BuiltinFn;
 
     fn from_proto(_: Self::Message) -> ProtoResult<Self>
     where
@@ -281,7 +293,7 @@ impl ToFromProto for ir::functions::map::MapExpr {
 }
 
 impl ToFromProto for ir::functions::sketch::SketchExpr {
-    type Message = proto::function::RsFunction;
+    type Message = proto::scalar_fn::BuiltinFn;
 
     fn from_proto(_: Self::Message) -> ProtoResult<Self>
     where
@@ -296,7 +308,7 @@ impl ToFromProto for ir::functions::sketch::SketchExpr {
 }
 
 impl ToFromProto for ir::functions::struct_::StructExpr {
-    type Message = proto::function::RsFunction;
+    type Message = proto::scalar_fn::BuiltinFn;
 
     fn from_proto(_: Self::Message) -> ProtoResult<Self>
     where
@@ -311,7 +323,7 @@ impl ToFromProto for ir::functions::struct_::StructExpr {
 }
 
 impl ToFromProto for ir::functions::partitioning::PartitioningExpr {
-    type Message = proto::function::RsFunction;
+    type Message = proto::scalar_fn::BuiltinFn;
 
     fn from_proto(_: Self::Message) -> ProtoResult<Self>
     where
@@ -325,9 +337,9 @@ impl ToFromProto for ir::functions::partitioning::PartitioningExpr {
     }
 }
 
-/// Converts a python UDF into a protobuf via the pickled callable and its args.
-impl ToFromProto for ir::functions::python::PythonUDF {
-    type Message = proto::function::PyFunction;
+/// Converts a legacy python UDF into a protobuf via the pickled callable and its args.
+impl ToFromProto for ir::functions::python::LegacyPythonUDF {
+    type Message = proto::scalar_fn::py_fn::LegacyFn;
 
     fn from_proto(message: Self::Message) -> ProtoResult<Self>
     where
@@ -352,6 +364,7 @@ impl ToFromProto for ir::functions::python::PythonUDF {
         // Convert the numeric fields back to their original types
         let concurrency = message.concurrency.map(|c| c as usize);
         let batch_size = message.batch_size.map(|b| b as usize);
+        let use_process = message.use_process;
 
         // Reconstruct the ResourceRequest from the flattened fields
         let resource_request = {
@@ -379,6 +392,7 @@ impl ToFromProto for ir::functions::python::PythonUDF {
             resource_request,
             batch_size,
             concurrency,
+            use_process,
         })
     }
 
@@ -389,7 +403,7 @@ impl ToFromProto for ir::functions::python::PythonUDF {
         let return_type = self.return_dtype.to_proto()?;
 
         // Convert all the python things.
-        let (callable, callalbe_init_args) = match &self.func {
+        let (callable, callable_init_args) = match &self.func {
             ir::functions::python::MaybeInitializedUDF::Initialized(py) => {
                 // no args, already have the callable
                 let callable = py.to_proto()?;
@@ -425,13 +439,14 @@ impl ToFromProto for ir::functions::python::PythonUDF {
             arity,
             return_type: Some(return_type),
             callable: Some(callable),
-            callable_init_args: Some(callalbe_init_args),
+            callable_init_args: Some(callable_init_args),
             callable_call_args: Some(callable_call_args),
             concurrency,
             batch_size,
             num_cpus,
             num_gpus,
             max_memory_bytes,
+            use_process: self.use_process,
         })
     }
 }
