@@ -9,8 +9,14 @@ use super::intermediate_op::{
 };
 use crate::{ops::NodeType, pipeline::NodeName, ExecutionTaskSpawner};
 
+#[derive(Debug)]
+enum SamplingMethod {
+    Fraction(f64),
+    Size(usize),
+}
+
 struct SampleParams {
-    fraction: f64,
+    sampling_method: SamplingMethod,
     with_replacement: bool,
     seed: Option<u64>,
 }
@@ -23,7 +29,17 @@ impl SampleOperator {
     pub fn new(fraction: f64, with_replacement: bool, seed: Option<u64>) -> Self {
         Self {
             params: Arc::new(SampleParams {
-                fraction,
+                sampling_method: SamplingMethod::Fraction(fraction),
+                with_replacement,
+                seed,
+            }),
+        }
+    }
+
+    pub fn new_size(size: usize, with_replacement: bool, seed: Option<u64>) -> Self {
+        Self {
+            params: Arc::new(SampleParams {
+                sampling_method: SamplingMethod::Size(size),
                 with_replacement,
                 seed,
             }),
@@ -45,11 +61,16 @@ impl IntermediateOperator for SampleOperator {
         task_spawner
             .spawn(
                 async move {
-                    let out = input.sample_by_fraction(
-                        params.fraction,
-                        params.with_replacement,
-                        params.seed,
-                    )?;
+                    let out = match &params.sampling_method {
+                        SamplingMethod::Fraction(fraction) => input.sample_by_fraction(
+                            *fraction,
+                            params.with_replacement,
+                            params.seed,
+                        )?,
+                        SamplingMethod::Size(size) => {
+                            input.sample_by_size(*size, params.with_replacement, params.seed)?
+                        }
+                    };
                     Ok((
                         state,
                         IntermediateOperatorResult::NeedMoreInput(Some(Arc::new(out))),
@@ -62,7 +83,10 @@ impl IntermediateOperator for SampleOperator {
 
     fn multiline_display(&self) -> Vec<String> {
         let mut res = vec![];
-        res.push(format!("Sample: {}", self.params.fraction));
+        match &self.params.sampling_method {
+            SamplingMethod::Fraction(fraction) => res.push(format!("Sample: {}", fraction)),
+            SamplingMethod::Size(size) => res.push(format!("Sample: {} rows", size)),
+        }
         res.push(format!(
             "With replacement = {}",
             self.params.with_replacement
