@@ -7,12 +7,7 @@ use tracing::instrument;
 use super::base::{
     StreamingSink, StreamingSinkExecuteResult, StreamingSinkFinalizeResult, StreamingSinkOutput,
 };
-use crate::{
-    dispatcher::{DispatchSpawner, RoundRobinDispatcher, UnorderedDispatcher},
-    ops::NodeType,
-    pipeline::NodeName,
-    ExecutionRuntimeContext, ExecutionTaskSpawner,
-};
+use crate::{ops::NodeType, pipeline::NodeName, ExecutionTaskSpawner};
 
 pub struct ConcatSink {}
 
@@ -56,16 +51,4 @@ impl StreamingSink for ConcatSink {
     fn max_concurrency(&self) -> usize {
         get_compute_pool_num_threads()
     }
-
-    fn dispatch_spawner(
-        &self,
-        _runtime_handle: &ExecutionRuntimeContext,
-        maintain_order: bool,
-    ) -> Arc<dyn DispatchSpawner> {
-        if maintain_order {
-            Arc::new(RoundRobinDispatcher::unbounded())
-        } else {
-            Arc::new(UnorderedDispatcher::unbounded())
-        }
-    }
 }
