@@ -9,12 +9,7 @@ use tracing::{instrument, Span};
 use super::base::{
     StreamingSink, StreamingSinkExecuteResult, StreamingSinkFinalizeResult, StreamingSinkOutput,
 };
-use crate::{
-    dispatcher::{DispatchSpawner, UnorderedDispatcher},
-    ops::NodeType,
-    pipeline::NodeName,
-    ExecutionRuntimeContext, ExecutionTaskSpawner,
-};
+use crate::{ops::NodeType, pipeline::NodeName, ExecutionTaskSpawner};
 
 pub(crate) struct LimitSinkState {
     // The remaining number of rows to skip
@@ -133,14 +128,4 @@ impl StreamingSink for LimitSink {
     fn max_concurrency(&self) -> usize {
         1
     }
-
-    fn dispatch_spawner(
-        &self,
-        _runtime_handle: &ExecutionRuntimeContext,
-        _maintain_order: bool,
-    ) -> Arc<dyn DispatchSpawner> {
-        // Limits are greedy, so we don't need to buffer any input.
-        // They are also not concurrent, so we don't need to worry about ordering.
-        Arc::new(UnorderedDispatcher::unbounded())
-    }
 }
