@@ -7,12 +7,7 @@ use tracing::{instrument, Span};
 use super::base::{
     StreamingSink, StreamingSinkExecuteResult, StreamingSinkFinalizeResult, StreamingSinkOutput,
 };
-use crate::{
-    dispatcher::{DispatchSpawner, UnorderedDispatcher},
-    ops::NodeType,
-    pipeline::NodeName,
-    ExecutionRuntimeContext, ExecutionTaskSpawner,
-};
+use crate::{ops::NodeType, pipeline::NodeName, ExecutionTaskSpawner};
 
 pub(crate) struct MonotonicallyIncreasingIdState {
     id_offset: u64,
@@ -125,12 +120,4 @@ impl StreamingSink for MonotonicallyIncreasingIdSink {
     fn max_concurrency(&self) -> usize {
         1
     }
-
-    fn dispatch_spawner(
-        &self,
-        _runtime_handle: &ExecutionRuntimeContext,
-        _maintain_order: bool,
-    ) -> Arc<dyn DispatchSpawner> {
-        Arc::new(UnorderedDispatcher::unbounded())
-    }
 }
