@@ -39,9 +39,9 @@ use crate::{
     ops::{
         self,
         join::{JoinOptions, JoinPredicate},
-        SetQuantifier, UnionStrategy,
+        Limit, Offset, SetQuantifier, UnionStrategy,
     },
-    optimization::OptimizerBuilder,
+    optimization::{OptimizerBuilder, OptimizerConfig},
     partitioning::{
         HashRepartitionConfig, IntoPartitionsConfig, RandomShuffleConfig, RepartitionSpec,
     },
@@ -325,7 +325,21 @@ impl LogicalPlanBuilder {
     }
 
     pub fn limit(&self, limit: u64, eager: bool) -> DaftResult<Self> {
-        let logical_plan: LogicalPlan = ops::Limit::new(self.plan.clone(), limit, eager).into();
+        self.limit_with_offset(limit, None, eager)
+    }
+
+    pub(crate) fn limit_with_offset(
+        &self,
+        limit: u64,
+        offset: Option<u64>,
+        eager: bool,
+    ) -> DaftResult<Self> {
+        let logical_plan: LogicalPlan = Limit::new(self.plan.clone(), limit, offset, eager).into();
+        Ok(self.with_new_plan(logical_plan))
+    }
+
+    pub fn offset(&self, offset: u64) -> DaftResult<Self> {
+        let logical_plan: LogicalPlan = Offset::new(self.plan.clone(), offset).into();
         Ok(self.with_new_plan(logical_plan))
     }
 
@@ -429,6 +443,11 @@ impl LogicalPlanBuilder {
         Ok(self.with_new_plan(logical_plan))
     }
 
+    pub fn into_batches(&self, batch_size: usize) -> DaftResult<Self> {
+        let logical_plan: LogicalPlan = ops::IntoBatches::new(self.plan.clone(), batch_size).into();
+        Ok(self.with_new_plan(logical_plan))
+    }
+
     /// Creates a logical scan operator by collapsing the plan to just its schema.
     #[cfg(feature = "python")]
     pub fn describe(&self) -> DaftResult<Self> {
@@ -805,6 +824,18 @@ impl LogicalPlanBuilder {
 
         std::thread::spawn(move || {
             let optimizer = OptimizerBuilder::default()
+                .when(
+                    cfg.as_ref()
+                        .map(|conf| conf.enable_strict_filter_pushdown)
+                        .unwrap_or(false),
+                    |builder| {
+                        builder.with_optimizer_config(OptimizerConfig {
+                            strict_pushdown: true,
+                            ..Default::default()
+                        })
+                    },
+                )
+                .with_default_optimizations()
                 .when(
                     !cfg.as_ref()
                         .is_some_and(|conf| conf.disable_join_reordering),
@@ -860,6 +891,18 @@ impl LogicalPlanBuilder {
         let unoptimized_plan = self.build();
 
         let optimizer = OptimizerBuilder::default()
+            .when(
+                cfg.as_ref()
+                    .map(|conf| conf.enable_strict_filter_pushdown)
+                    .unwrap_or(false),
+                |builder| {
+                    builder.with_optimizer_config(OptimizerConfig {
+                        strict_pushdown: true,
+                        ..Default::default()
+                    })
+                },
+            )
+            .with_default_optimizations()
             .when(
                 !cfg.as_ref()
                     .is_some_and(|conf| conf.disable_join_reordering),
@@ -1049,6 +1092,16 @@ impl PyLogicalPlanBuilder {
         Ok(self.builder.limit(limit as u64, eager)?.into())
     }
 
+    pub fn offset(&self, offset: i64) -> PyResult<Self> {
+        if offset < 0 {
+            return Err(pyo3::exceptions::PyValueError::new_err(format!(
+                "OFFSET <n> must be greater than or equal to 0, instead got: {}",
+                offset
+            )));
+        }
+        Ok(self.builder.offset(offset as u64)?.into())
+    }
+
     pub fn shard(&self, strategy: String, world_size: i64, rank: i64) -> PyResult<Self> {
         Ok(self.builder.shard(strategy, world_size, rank)?.into())
     }
@@ -1111,6 +1164,10 @@ impl PyLogicalPlanBuilder {
         Ok(self.builder.into_partitions(num_partitions)?.into())
     }
 
+    pub fn into_batches(&self, batch_size: usize) -> PyResult<Self> {
+        Ok(self.builder.into_batches(batch_size)?.into())
+    }
+
     pub fn describe(&self) -> PyResult<Self> {
         Ok(self.builder.describe()?.into())
     }
