@@ -220,17 +220,22 @@ Project1 --> Limit0
             bottom_up: false,
             subgraph_options: Some(SubgraphOptions {
                 name: "Optimized Logical Plan".to_string(),
-                subgraph_id: "optimized".to_string(),
-                metadata: None,
+                subgraph_id: "optimized<daft::LogicalPlan>".to_string(),
+                metadata: Some(
+                    "{'rules': 'RuleBatch<daft::PushDownLimit>, RuleBatch<daft::EnrichWithStats>'}"
+                        .to_string(),
+                ),
             }),
         };
 
         let mermaid_repr = plan.repr_mermaid(opts);
-        let expected = r#"subgraph optimized["Optimized Logical Plan"]
+        let expected = r#"subgraph optimized__daft::LogicalPlan__["Optimized Logical Plan"]
 direction TB
-optimizedSource0["PlaceHolder:
+optimized__daft::LogicalPlan___metadata["{'rules': 'RuleBatch&lt;daft::PushDownLimit&gt;, RuleBatch&lt;daft::EnrichWithStats&gt;'}"]
+optimized__daft::LogicalPlan__Source0["PlaceHolder:
 Num partitions = 0
 Output schema = text#Utf8, id#Int32"]
+optimized__daft::LogicalPlan___metadata ~~~ optimized__daft::LogicalPlan__Source0
 end
 "#;
         assert_eq!(mermaid_repr, expected);
