@@ -26,9 +26,11 @@ pub enum LogicalPlan {
     Source(Source),
     Shard(Shard),
     Project(Project),
-    ActorPoolProject(ActorPoolProject),
+    UDFProject(UDFProject),
     Filter(Filter),
+    IntoBatches(IntoBatches),
     Limit(Limit),
+    Offset(Offset),
     Explode(Explode),
     Unpivot(Unpivot),
     Sort(Sort),
@@ -83,6 +85,40 @@ impl SubqueryAlias {
     }
 }
 
+/// Return value of [LogicalPlan::required_columns]
+pub(crate) struct RequiredCols {
+    /// Base required columns for an operator. For operators with 2 sides
+    /// (e.g. join, concat), this is the required columns for the left side.
+    pub(crate) required_cols: IndexSet<String>,
+    /// Optional right side columns for an operator. For operators with 2 sides
+    /// (e.g. join, concat), this is the optional columns for the right side.
+    /// For other operators, this is None.
+    pub(crate) opt_right_cols: Option<IndexSet<String>>,
+}
+
+impl RequiredCols {
+    pub fn new(required_cols: IndexSet<String>, opt_right_cols: Option<IndexSet<String>>) -> Self {
+        Self {
+            required_cols,
+            opt_right_cols,
+        }
+    }
+
+    /// Unwrap and return the required (left) side columns. Intended for 1-side ops.
+    pub fn single(self) -> IndexSet<String> {
+        self.required_cols
+    }
+
+    /// Unwrap and return the required (left) and optional (right) side columns. Intended for 2-side ops.
+    pub fn double(self) -> (IndexSet<String>, IndexSet<String>) {
+        if let Some(opt_right_cols) = self.opt_right_cols {
+            (self.required_cols, opt_right_cols)
+        } else {
+            panic!("Expected 2-side plan node (e.g. join, concat), but got 1-side plan node")
+        }
+    }
+}
+
 impl LogicalPlan {
     pub fn arced(self) -> Arc<Self> {
         Arc::new(self)
@@ -95,11 +131,13 @@ impl LogicalPlan {
             Self::Project(Project {
                 projected_schema, ..
             }) => projected_schema.clone(),
-            Self::ActorPoolProject(ActorPoolProject {
+            Self::UDFProject(UDFProject {
                 projected_schema, ..
             }) => projected_schema.clone(),
             Self::Filter(Filter { input, .. }) => input.schema(),
+            Self::IntoBatches(IntoBatches { input, .. }) => input.schema(),
             Self::Limit(Limit { input, .. }) => input.schema(),
+            Self::Offset(Offset { input, .. }) => input.schema(),
             Self::Explode(Explode {
                 exploded_schema, ..
             }) => exploded_schema.clone(),
@@ -124,35 +162,47 @@ impl LogicalPlan {
         }
     }
 
-    pub fn required_columns(&self) -> Vec<IndexSet<String>> {
+    pub(crate) fn required_columns(&self) -> RequiredCols {
         // TODO: https://github.com/Eventual-Inc/Daft/pull/1288#discussion_r1307820697
         match self {
-            Self::Shard(..) => vec![IndexSet::new()],
-            Self::Limit(..) => vec![IndexSet::new()],
-            Self::Sample(..) => vec![IndexSet::new()],
-            Self::MonotonicallyIncreasingId(..) => vec![IndexSet::new()],
-            Self::Concat(..) => vec![IndexSet::new(), IndexSet::new()],
+            Self::Shard(..)
+            | Self::Limit(..)
+            | Self::IntoBatches(..)
+            | Self::Offset(..)
+            | Self::Sample(..)
+            | Self::MonotonicallyIncreasingId(..) => RequiredCols::new(IndexSet::new(), None),
+            Self::Concat(..) => RequiredCols::new(IndexSet::new(), Some(IndexSet::new())),
             Self::Project(projection) => {
                 let res = projection
                     .projection
                     .iter()
                     .flat_map(get_required_columns)
                     .collect();
-                vec![res]
+                RequiredCols::new(res, None)
             }
-            Self::ActorPoolProject(ActorPoolProject { projection, .. }) => {
-                let res = projection.iter().flat_map(get_required_columns).collect();
-                vec![res]
+            Self::UDFProject(UDFProject {
+                project,
+                passthrough_columns,
+                ..
+            }) => {
+                let mut res = passthrough_columns
+                    .iter()
+                    .flat_map(get_required_columns)
+                    .collect::<IndexSet<_>>();
+
+                res.extend(get_required_columns(project).into_iter());
+                RequiredCols::new(res, None)
             }
-            Self::Filter(filter) => {
-                vec![get_required_columns(&filter.predicate)
+            Self::Filter(filter) => RequiredCols::new(
+                get_required_columns(&filter.predicate)
                     .iter()
                     .cloned()
-                    .collect()]
-            }
+                    .collect(),
+                None,
+            ),
             Self::Sort(sort) => {
                 let res = sort.sort_by.iter().flat_map(get_required_columns).collect();
-                vec![res]
+                RequiredCols::new(res, None)
             }
             Self::Repartition(repartition) => {
                 let res = repartition
@@ -161,7 +211,7 @@ impl LogicalPlan {
                     .iter()
                     .flat_map(get_required_columns)
                     .collect();
-                vec![res]
+                RequiredCols::new(res, None)
             }
             Self::Explode(explode) => {
                 let res = explode
@@ -169,7 +219,7 @@ impl LogicalPlan {
                     .iter()
                     .flat_map(get_required_columns)
                     .collect();
-                vec![res]
+                RequiredCols::new(res, None)
             }
             Self::Unpivot(unpivot) => {
                 let res = unpivot
@@ -178,12 +228,12 @@ impl LogicalPlan {
                     .chain(unpivot.values.iter())
                     .flat_map(get_required_columns)
                     .collect();
-                vec![res]
+                RequiredCols::new(res, None)
             }
             Self::Distinct(distinct) => {
                 if let Some(on) = &distinct.columns {
                     let res = on.iter().flat_map(get_required_columns).collect();
-                    vec![res]
+                    RequiredCols::new(res, None)
                 } else {
                     let res = distinct
                         .input
@@ -191,7 +241,7 @@ impl LogicalPlan {
                         .field_names()
                         .map(ToString::to_string)
                         .collect();
-                    vec![res]
+                    RequiredCols::new(res, None)
                 }
             }
             Self::Aggregate(aggregate) => {
@@ -202,7 +252,7 @@ impl LogicalPlan {
                     .flat_map(|e| get_required_columns(&e))
                     .chain(aggregate.groupby.iter().flat_map(get_required_columns))
                     .collect();
-                vec![res]
+                RequiredCols::new(res, None)
             }
             Self::Pivot(pivot) => {
                 let res = pivot
@@ -212,7 +262,7 @@ impl LogicalPlan {
                     .chain(std::iter::once(&pivot.value_column))
                     .flat_map(get_required_columns)
                     .collect();
-                vec![res]
+                RequiredCols::new(res, None)
             }
             Self::Join(join) => {
                 let mut left = IndexSet::new();
@@ -240,10 +290,10 @@ impl LogicalPlan {
                     })
                     .unwrap();
                 }
-                vec![left, right]
+                RequiredCols::new(left, Some(right))
             }
-            Self::Intersect(_) => vec![IndexSet::new(), IndexSet::new()],
-            Self::Union(_) => vec![IndexSet::new(), IndexSet::new()],
+            Self::Intersect(_) => RequiredCols::new(IndexSet::new(), Some(IndexSet::new())),
+            Self::Union(_) => RequiredCols::new(IndexSet::new(), Some(IndexSet::new())),
             Self::Source(_) => todo!(),
             Self::Sink(_) => todo!(),
             Self::SubqueryAlias(SubqueryAlias { input, .. }) => input.required_columns(),
@@ -255,7 +305,7 @@ impl LogicalPlan {
                     .chain(window.window_spec.order_by.iter())
                     .flat_map(get_required_columns)
                     .collect();
-                vec![res]
+                RequiredCols::new(res, None)
             }
             Self::TopN(top_n) => {
                 let res = top_n
@@ -263,7 +313,7 @@ impl LogicalPlan {
                     .iter()
                     .flat_map(get_required_columns)
                     .collect();
-                vec![res]
+                RequiredCols::new(res, None)
             }
         }
     }
@@ -273,9 +323,11 @@ impl LogicalPlan {
             Self::Source(..) => "Source",
             Self::Shard(..) => "Shard",
             Self::Project(..) => "Project",
-            Self::ActorPoolProject(..) => "ActorPoolProject",
+            Self::UDFProject(..) => "UDFProject",
             Self::Filter(..) => "Filter",
+            Self::IntoBatches(..) => "IntoBatches",
             Self::Limit(..) => "Limit",
+            Self::Offset(..) => "Offset",
             Self::Explode(..) => "Explode",
             Self::Unpivot(..) => "Unpivot",
             Self::Sort(..) => "Sort",
@@ -301,9 +353,11 @@ impl LogicalPlan {
             Self::Source(Source { stats_state, .. })
             | Self::Shard(Shard { stats_state, .. })
             | Self::Project(Project { stats_state, .. })
-            | Self::ActorPoolProject(ActorPoolProject { stats_state, .. })
+            | Self::UDFProject(UDFProject { stats_state, .. })
             | Self::Filter(Filter { stats_state, .. })
+            | Self::IntoBatches(IntoBatches { stats_state, .. })
             | Self::Limit(Limit { stats_state, .. })
+            | Self::Offset(Offset { stats_state, .. })
             | Self::Explode(Explode { stats_state, .. })
             | Self::Unpivot(Unpivot { stats_state, .. })
             | Self::Sort(Sort { stats_state, .. })
@@ -318,14 +372,11 @@ impl LogicalPlan {
             | Self::MonotonicallyIncreasingId(MonotonicallyIncreasingId { stats_state, .. })
             | Self::Window(Window { stats_state, .. })
             | Self::TopN(TopN { stats_state, .. }) => stats_state,
-            Self::Intersect(_) => {
-                panic!("Intersect nodes should be optimized away before stats are materialized")
-            }
-            Self::Union(_) => {
-                panic!("Union nodes should be optimized away before stats are materialized")
-            }
-            Self::SubqueryAlias(_) => {
-                panic!("Alias nodes should be optimized away before stats are materialized")
+            Self::Intersect(_) | Self::Union(_) | Self::SubqueryAlias(_) => {
+                panic!(
+                    "{} nodes should be optimized away before stats are materialized",
+                    self.name()
+                )
             }
         }
     }
@@ -341,9 +392,11 @@ impl LogicalPlan {
             Self::Source(plan) => Self::Source(plan.with_materialized_stats()),
             Self::Shard(plan) => Self::Shard(plan.with_materialized_stats()),
             Self::Project(plan) => Self::Project(plan.with_materialized_stats()),
-            Self::ActorPoolProject(plan) => Self::ActorPoolProject(plan.with_materialized_stats()),
+            Self::UDFProject(plan) => Self::UDFProject(plan.with_materialized_stats()),
             Self::Filter(plan) => Self::Filter(plan.with_materialized_stats()),
+            Self::IntoBatches(plan) => Self::IntoBatches(plan.with_materialized_stats()),
             Self::Limit(plan) => Self::Limit(plan.with_materialized_stats()),
+            Self::Offset(plan) => Self::Offset(plan.with_materialized_stats()),
             Self::Explode(plan) => Self::Explode(plan.with_materialized_stats()),
             Self::Unpivot(plan) => Self::Unpivot(plan.with_materialized_stats()),
             Self::Sort(plan) => Self::Sort(plan.with_materialized_stats()),
@@ -352,15 +405,6 @@ impl LogicalPlan {
             Self::Aggregate(plan) => Self::Aggregate(plan.with_materialized_stats()),
             Self::Pivot(plan) => Self::Pivot(plan.with_materialized_stats()),
             Self::Concat(plan) => Self::Concat(plan.with_materialized_stats()),
-            Self::Intersect(_) => {
-                panic!("Intersect should be optimized away before stats are derived")
-            }
-            Self::Union(_) => {
-                panic!("Union should be optimized away before stats are derived")
-            }
-            Self::SubqueryAlias(_) => {
-                panic!("Alias should be optimized away before stats are derived")
-            }
             Self::Join(plan) => Self::Join(plan.with_materialized_stats()),
             Self::Sink(plan) => Self::Sink(plan.with_materialized_stats()),
             Self::Sample(plan) => Self::Sample(plan.with_materialized_stats()),
@@ -369,6 +413,12 @@ impl LogicalPlan {
             }
             Self::Window(plan) => Self::Window(plan.with_materialized_stats()),
             Self::TopN(plan) => Self::TopN(plan.with_materialized_stats()),
+            Self::Intersect(_) | Self::Union(_) | Self::SubqueryAlias(_) => {
+                panic!(
+                    "{} should be optimized away before stats are derived",
+                    self.name()
+                )
+            }
         }
     }
 
@@ -377,9 +427,11 @@ impl LogicalPlan {
             Self::Source(source) => source.multiline_display(),
             Self::Shard(shard) => shard.multiline_display(),
             Self::Project(projection) => projection.multiline_display(),
-            Self::ActorPoolProject(projection) => projection.multiline_display(),
+            Self::UDFProject(projection) => projection.multiline_display(),
             Self::Filter(filter) => filter.multiline_display(),
+            Self::IntoBatches(into_batches) => into_batches.multiline_display(),
             Self::Limit(limit) => limit.multiline_display(),
+            Self::Offset(offset) => offset.multiline_display(),
             Self::Explode(explode) => explode.multiline_display(),
             Self::Unpivot(unpivot) => unpivot.multiline_display(),
             Self::Sort(sort) => sort.multiline_display(),
@@ -407,9 +459,11 @@ impl LogicalPlan {
             Self::Source(..) => vec![],
             Self::Shard(Shard { input, .. }) => vec![input],
             Self::Project(Project { input, .. }) => vec![input],
-            Self::ActorPoolProject(ActorPoolProject { input, .. }) => vec![input],
+            Self::UDFProject(UDFProject { input, .. }) => vec![input],
             Self::Filter(Filter { input, .. }) => vec![input],
+            Self::IntoBatches(IntoBatches { input, .. }) => vec![input],
             Self::Limit(Limit { input, .. }) => vec![input],
+            Self::Offset(Offset { input, .. }) => vec![input],
             Self::Explode(Explode { input, .. }) => vec![input],
             Self::Unpivot(Unpivot { input, .. }) => vec![input],
             Self::Sort(Sort { input, .. }) => vec![input],
@@ -440,9 +494,11 @@ impl LogicalPlan {
                 Self::Project(Project { projection, .. }) => Self::Project(Project::try_new(
                         input.clone(), projection.clone(),
                     ).unwrap()),
-                Self::ActorPoolProject(ActorPoolProject {projection, ..}) => Self::ActorPoolProject(ActorPoolProject::try_new(input.clone(), projection.clone()).unwrap()),
+                Self::UDFProject(UDFProject {project, passthrough_columns, ..}) => Self::UDFProject(UDFProject::try_new(input.clone(), project.clone(), passthrough_columns.clone()).unwrap()),
                 Self::Filter(Filter { predicate, .. }) => Self::Filter(Filter::try_new(input.clone(), predicate.clone()).unwrap()),
-                Self::Limit(Limit { limit, eager, .. }) => Self::Limit(Limit::new(input.clone(), *limit, *eager)),
+                Self::IntoBatches(IntoBatches { batch_size, .. }) => Self::IntoBatches(IntoBatches::new(input.clone(), *batch_size)),
+                Self::Limit(Limit { limit, offset, eager, .. }) => Self::Limit(Limit::new(input.clone(), *limit, *offset, *eager)),
+                Self::Offset(Offset { offset, .. }) => Self::Offset(Offset::new(input.clone(), *offset)),
                 Self::Explode(Explode { to_explode, .. }) => Self::Explode(Explode::try_new(input.clone(), to_explode.clone()).unwrap()),
                 Self::Sort(Sort { sort_by, descending, nulls_first, .. }) => Self::Sort(Sort::try_new(input.clone(), sort_by.clone(), descending.clone(), nulls_first.clone()).unwrap()),
                 Self::Repartition(Repartition {  repartition_spec: scheme_config, .. }) => Self::Repartition(Repartition::new(input.clone(), scheme_config.clone())),
@@ -461,13 +517,13 @@ impl LogicalPlan {
                     aliases.clone(),
                     window_spec.clone(),
                 ).unwrap()),
-                Self::TopN(TopN { sort_by, descending, nulls_first, limit, .. }) => Self::TopN(TopN::try_new(
-                    input.clone(), sort_by.clone(), descending.clone(), nulls_first.clone(), *limit
+                Self::TopN(TopN { sort_by, descending, nulls_first, limit, offset, .. }) => Self::TopN(TopN::try_new(
+                    input.clone(), sort_by.clone(), descending.clone(), nulls_first.clone(), *limit, *offset
                 ).unwrap()),
-                Self::Concat(_) => panic!("Concat ops should never have only one input, but got one"),
-                Self::Intersect(_) => panic!("Intersect ops should never have only one input, but got one"),
-                Self::Union(_) => panic!("Union ops should never have only one input, but got one"),
-                Self::Join(_) => panic!("Join ops should never have only one input, but got one"),
+                Self::Concat(_)
+                | Self::Intersect(_)
+                | Self::Union(_)
+                | Self::Join(_) => panic!("{} ops should never have only one input, but got one", input.name()),
             },
             [input1, input2] => match self {
                 Self::Source(_) => panic!("Source nodes don't have children, with_new_children() should never be called for Source ops"),
@@ -592,9 +648,11 @@ impl LogicalPlan {
             Self::Source(Source { plan_id, .. })
             | Self::Shard(Shard { plan_id, .. })
             | Self::Project(Project { plan_id, .. })
-            | Self::ActorPoolProject(ActorPoolProject { plan_id, .. })
+            | Self::UDFProject(UDFProject { plan_id, .. })
             | Self::Filter(Filter { plan_id, .. })
+            | Self::IntoBatches(IntoBatches { plan_id, .. })
             | Self::Limit(Limit { plan_id, .. })
+            | Self::Offset(Offset { plan_id, .. })
             | Self::Explode(Explode { plan_id, .. })
             | Self::Unpivot(Unpivot { plan_id, .. })
             | Self::Sort(Sort { plan_id, .. })
@@ -620,9 +678,11 @@ impl LogicalPlan {
             Self::Source(Source { node_id, .. })
             | Self::Shard(Shard { node_id, .. })
             | Self::Project(Project { node_id, .. })
-            | Self::ActorPoolProject(ActorPoolProject { node_id, .. })
+            | Self::UDFProject(UDFProject { node_id, .. })
             | Self::Filter(Filter { node_id, .. })
+            | Self::IntoBatches(IntoBatches { node_id, .. })
             | Self::Limit(Limit { node_id, .. })
+            | Self::Offset(Offset { node_id, .. })
             | Self::Explode(Explode { node_id, .. })
             | Self::Unpivot(Unpivot { node_id, .. })
             | Self::Sort(Sort { node_id, .. })
@@ -649,11 +709,13 @@ impl LogicalPlan {
             Self::Source(source) => Self::Source(source.with_plan_id(plan_id)),
             Self::Shard(shard) => Self::Shard(shard.with_plan_id(plan_id)),
             Self::Project(project) => Self::Project(project.with_plan_id(plan_id)),
-            Self::ActorPoolProject(project) => {
-                Self::ActorPoolProject(project.with_plan_id(plan_id))
-            }
+            Self::UDFProject(project) => Self::UDFProject(project.with_plan_id(plan_id)),
             Self::Filter(filter) => Self::Filter(filter.with_plan_id(plan_id)),
+            Self::IntoBatches(into_batches) => {
+                Self::IntoBatches(into_batches.with_plan_id(plan_id))
+            }
             Self::Limit(limit) => Self::Limit(limit.with_plan_id(plan_id)),
+            Self::Offset(offset) => Self::Offset(offset.with_plan_id(plan_id)),
             Self::Explode(explode) => Self::Explode(explode.with_plan_id(plan_id)),
             Self::Unpivot(unpivot) => Self::Unpivot(unpivot.with_plan_id(plan_id)),
             Self::Sort(sort) => Self::Sort(sort.with_plan_id(plan_id)),
@@ -682,11 +744,13 @@ impl LogicalPlan {
             Self::Source(source) => Self::Source(source.with_node_id(node_id)),
             Self::Shard(shard) => Self::Shard(shard.with_node_id(node_id)),
             Self::Project(project) => Self::Project(project.with_node_id(node_id)),
-            Self::ActorPoolProject(project) => {
-                Self::ActorPoolProject(project.with_node_id(node_id))
-            }
+            Self::UDFProject(project) => Self::UDFProject(project.with_node_id(node_id)),
             Self::Filter(filter) => Self::Filter(filter.with_node_id(node_id)),
+            Self::IntoBatches(into_batches) => {
+                Self::IntoBatches(into_batches.with_node_id(node_id))
+            }
             Self::Limit(limit) => Self::Limit(limit.with_node_id(node_id)),
+            Self::Offset(offset) => Self::Offset(offset.with_node_id(node_id)),
             Self::Explode(explode) => Self::Explode(explode.with_node_id(node_id)),
             Self::Unpivot(unpivot) => Self::Unpivot(unpivot.with_node_id(node_id)),
             Self::Sort(sort) => Self::Sort(sort.with_node_id(node_id)),
@@ -795,7 +859,9 @@ impl_from_data_struct_for_logical_plan!(Source);
 impl_from_data_struct_for_logical_plan!(Shard);
 impl_from_data_struct_for_logical_plan!(Project);
 impl_from_data_struct_for_logical_plan!(Filter);
+impl_from_data_struct_for_logical_plan!(IntoBatches);
 impl_from_data_struct_for_logical_plan!(Limit);
+impl_from_data_struct_for_logical_plan!(Offset);
 impl_from_data_struct_for_logical_plan!(Explode);
 impl_from_data_struct_for_logical_plan!(Unpivot);
 impl_from_data_struct_for_logical_plan!(Sort);
