@@ -1,12 +1,13 @@
-mod actor_pool_project;
 mod agg;
 mod concat;
 mod distinct;
 mod explode;
 mod filter;
+mod into_batches;
 pub mod join;
 mod limit;
 mod monotonically_increasing_id;
+mod offset;
 mod pivot;
 mod project;
 mod repartition;
@@ -18,18 +19,20 @@ mod sort;
 mod source;
 mod summarize;
 mod top_n;
+mod udf;
 mod unpivot;
 mod window;
 
-pub use actor_pool_project::ActorPoolProject;
 pub use agg::Aggregate;
 pub use concat::Concat;
 pub use distinct::Distinct;
 pub use explode::Explode;
 pub use filter::Filter;
+pub use into_batches::IntoBatches;
 pub use join::Join;
 pub use limit::Limit;
 pub use monotonically_increasing_id::MonotonicallyIncreasingId;
+pub use offset::Offset;
 pub use pivot::Pivot;
 pub use project::Project;
 pub use repartition::Repartition;
@@ -41,5 +44,6 @@ pub use sort::Sort;
 pub use source::Source;
 pub use summarize::summarize;
 pub use top_n::TopN;
+pub use udf::UDFProject;
 pub use unpivot::Unpivot;
 pub use window::Window;
