@@ -4,8 +4,10 @@ use common_error::DaftResult;
 use common_treenode::{Transformed, TreeNode, TreeNodeRecursion};
 use daft_core::prelude::*;
 use daft_dsl::{
-    functions::FunctionArgs, optimization, resolved_col, AggExpr, ApproxPercentileParams, Column,
-    Expr, ExprRef,
+    functions::{scalar::ScalarFn, FunctionArgs},
+    optimization,
+    python_udf::{PyScalarFn, RowWisePyFn},
+    resolved_col, AggExpr, ApproxPercentileParams, Column, Expr, ExprRef,
 };
 use indexmap::{IndexMap, IndexSet};
 use itertools::Itertools;
@@ -446,7 +448,7 @@ fn replace_column_with_semantic_id(
                     )
                 }
             }
-            Expr::ScalarFunction(func) => {
+            Expr::ScalarFn(ScalarFn::Builtin(func)) => {
                 let mut func = func.clone();
                 let transforms = func
                     .inputs
@@ -465,7 +467,7 @@ fn replace_column_with_semantic_id(
                         .map(|t| t.map(|t| t.data.clone()))
                         .collect::<Vec<_>>();
                     func.inputs = FunctionArgs::new_unchecked(inputs);
-                    Transformed::yes(Expr::ScalarFunction(func).into())
+                    Transformed::yes(func.into())
                 }
             }
             Expr::InSubquery(expr, subquery) => {
@@ -477,6 +479,38 @@ fn replace_column_with_semantic_id(
                     Transformed::yes(Expr::InSubquery(expr.data, subquery.clone()).into())
                 }
             }
+            Expr::ScalarFn(ScalarFn::Python(PyScalarFn::RowWise(RowWisePyFn {
+                function_name: name,
+                inner: func,
+                return_dtype,
+                original_args,
+                args: children,
+            }))) => {
+                let transforms = children
+                    .iter()
+                    .map(|e| {
+                        replace_column_with_semantic_id(e.clone(), subexprs_to_replace, schema)
+                    })
+                    .collect::<Vec<_>>();
+
+                if transforms.iter().all(|e| !e.transformed) {
+                    Transformed::no(e)
+                } else {
+                    let new_children = transforms
+                        .iter()
+                        .map(|t| t.data.clone())
+                        .collect::<Vec<_>>();
+                    Transformed::yes(Arc::new(Expr::ScalarFn(ScalarFn::Python(
+                        PyScalarFn::RowWise(RowWisePyFn {
+                            function_name: name.clone(),
+                            inner: func.clone(),
+                            return_dtype: return_dtype.clone(),
+                            original_args: original_args.clone(),
+                            args: new_children,
+                        }),
+                    ))))
+                }
+            }
         }
     }
 }
