@@ -7,6 +7,7 @@ use crate::{partitioning::RepartitionSpec, stats::StatsState, LogicalPlan};
 #[derive(Clone, Debug, PartialEq, Eq, Hash, Serialize, Deserialize)]
 pub struct Repartition {
     pub plan_id: Option<usize>,
+    pub node_id: Option<usize>,
     // Upstream node.
     pub input: Arc<LogicalPlan>,
     pub repartition_spec: RepartitionSpec,
@@ -17,6 +18,7 @@ impl Repartition {
     pub(crate) fn new(input: Arc<LogicalPlan>, repartition_spec: RepartitionSpec) -> Self {
         Self {
             plan_id: None,
+            node_id: None,
             input,
             repartition_spec,
             stats_state: StatsState::NotMaterialized,
@@ -28,6 +30,11 @@ impl Repartition {
         self
     }
 
+    pub fn with_node_id(mut self, node_id: usize) -> Self {
+        self.node_id = Some(node_id);
+        self
+    }
+
     pub(crate) fn with_materialized_stats(mut self) -> Self {
         // Repartitioning does not affect cardinality.
         let input_stats = self.input.materialized_stats();
