@@ -21,6 +21,7 @@ pub fn dummy_scan_operator_with_size(
         schema,
         num_scan_tasks: 1,
         num_rows_per_task,
+        supports_count_pushdown_flag: false,
     }))
 }
 
@@ -36,3 +37,17 @@ pub fn dummy_scan_node_with_pushdowns(
 ) -> LogicalPlanBuilder {
     LogicalPlanBuilder::table_scan(scan_op, Some(pushdowns)).unwrap()
 }
+
+/// Create a dummy scan operator for aggregation tests.
+pub fn dummy_scan_operator_for_aggregation(
+    fields: Vec<Field>,
+    supports_count_pushdown_flag: bool,
+) -> ScanOperatorRef {
+    let schema: Arc<Schema> = Arc::new(Schema::new(fields));
+    ScanOperatorRef(Arc::new(DummyScanOperator {
+        schema,
+        num_scan_tasks: 1,
+        num_rows_per_task: None,
+        supports_count_pushdown_flag: supports_count_pushdown_flag,
+    }))
+}
