@@ -4,10 +4,10 @@ use std::{
     sync::Arc,
 };
 
-use arrow2::io::parquet::read::{column_iter_to_arrays, schema::infer_schema_with_options};
+use arrow2::io::parquet::read::column_iter_to_arrays;
 use common_error::DaftResult;
 use common_runtime::{combine_stream, get_io_runtime};
-use daft_core::{prelude::*, utils::arrow::cast_array_for_daft_if_needed};
+use daft_core::prelude::*;
 use daft_dsl::{expr::bound_expr::BoundExpr, ExprRef};
 use daft_io::{IOClient, IOStatsRef};
 use daft_recordbatch::RecordBatch;
@@ -22,7 +22,7 @@ use snafu::ResultExt;
 use tokio_stream::wrappers::ReceiverStream;
 
 use crate::{
-    determine_parquet_parallelism,
+    determine_parquet_parallelism, infer_arrow_schema_from_metadata,
     metadata::read_parquet_metadata,
     read::ParquetSchemaInferenceOptions,
     read_planner::{CoalescePass, RangesContainer, ReadPlanner, SplitLargeRequestPass},
@@ -211,12 +211,8 @@ impl ParquetReaderBuilder {
         io_stats: Option<IOStatsRef>,
         field_id_mapping: Option<Arc<BTreeMap<i32, Field>>>,
     ) -> super::Result<Self> {
-        // TODO(sammy): We actually don't need this since we can do negative offsets when reading the metadata
-        let size = io_client
-            .single_url_get_size(uri.into(), io_stats.clone())
-            .await?;
         let metadata =
-            read_parquet_metadata(uri, size, io_client, io_stats, field_id_mapping).await?;
+            read_parquet_metadata(uri, None, io_client, io_stats, field_id_mapping, None).await?;
         Ok(Self {
             uri: uri.into(),
             metadata,
@@ -273,11 +269,10 @@ impl ParquetReaderBuilder {
 
     pub fn build(self) -> super::Result<ParquetFileReader> {
         let options = self.schema_inference_options.into();
-        let mut arrow_schema = infer_schema_with_options(&self.metadata, Some(options)).context(
-            UnableToParseSchemaFromMetadataSnafu {
+        let mut arrow_schema = infer_arrow_schema_from_metadata(&self.metadata, Some(options))
+            .context(UnableToParseSchemaFromMetadataSnafu {
                 path: self.uri.clone(),
-            },
-        )?;
+            })?;
 
         if let Some(names_to_keep) = self.selected_columns {
             arrow_schema
@@ -678,12 +673,7 @@ impl ParquetFileReader {
                                     }
                                     all_arrays
                                         .into_iter()
-                                        .map(|a| {
-                                            Series::try_from((
-                                                field.name.as_str(),
-                                                cast_array_for_daft_if_needed(a),
-                                            ))
-                                        })
+                                        .map(|a| Series::try_from((field.name.as_str(), a)))
                                         .collect::<DaftResult<Vec<Series>>>()
                                 })();
 
