@@ -25,13 +25,12 @@ use daft_functions_list::{count_distinct, distinct};
 use daft_logical_plan::{
     logical_plan::LogicalPlan,
     ops::{
-        ActorPoolProject as LogicalActorPoolProject, Aggregate as LogicalAggregate,
-        Distinct as LogicalDistinct, Explode as LogicalExplode, Filter as LogicalFilter,
-        Join as LogicalJoin, Limit as LogicalLimit,
-        MonotonicallyIncreasingId as LogicalMonotonicallyIncreasingId, Pivot as LogicalPivot,
-        Project as LogicalProject, Repartition as LogicalRepartition, Sample as LogicalSample,
-        Sink as LogicalSink, Sort as LogicalSort, Source, TopN as LogicalTopN,
-        Unpivot as LogicalUnpivot,
+        Aggregate as LogicalAggregate, Distinct as LogicalDistinct, Explode as LogicalExplode,
+        Filter as LogicalFilter, IntoBatches as LogicalIntoBatches, Join as LogicalJoin,
+        Limit as LogicalLimit, MonotonicallyIncreasingId as LogicalMonotonicallyIncreasingId,
+        Pivot as LogicalPivot, Project as LogicalProject, Repartition as LogicalRepartition,
+        Sample as LogicalSample, Sink as LogicalSink, Sort as LogicalSort, Source,
+        TopN as LogicalTopN, UDFProject as LogicalUDFProject, Unpivot as LogicalUnpivot,
     },
     partitioning::{
         ClusteringSpec, HashClusteringConfig, RangeClusteringConfig, UnknownClusteringConfig,
@@ -123,13 +122,28 @@ pub(super) fn translate_single_logical_node(
                     .arced(),
             )
         }
-        LogicalPlan::ActorPoolProject(LogicalActorPoolProject { projection, .. }) => {
+        LogicalPlan::UDFProject(LogicalUDFProject {
+            project,
+            passthrough_columns,
+            udf_properties,
+            ..
+        }) => {
             let input_physical = physical_children.pop().expect("requires 1 input");
-            Ok(PhysicalPlan::ActorPoolProject(ActorPoolProject::try_new(
-                input_physical,
-                projection.clone(),
-            )?)
-            .arced())
+            let projection = passthrough_columns
+                .iter()
+                .chain(std::iter::once(&project.clone()))
+                .cloned()
+                .collect();
+            if udf_properties.is_actor_pool_udf() {
+                Ok(PhysicalPlan::ActorPoolProject(ActorPoolProject::try_new(
+                    input_physical,
+                    projection,
+                    udf_properties.clone(),
+                )?)
+                .arced())
+            } else {
+                Ok(PhysicalPlan::Project(Project::try_new(input_physical, projection)?).arced())
+            }
         }
         LogicalPlan::Filter(LogicalFilter {
             predicate, input, ..
@@ -143,18 +157,31 @@ pub(super) fn translate_single_logical_node(
             ))
             .arced())
         }
-        LogicalPlan::Limit(LogicalLimit { limit, eager, .. }) => {
+        LogicalPlan::IntoBatches(LogicalIntoBatches { .. }) => Err(DaftError::NotImplemented(
+            "IntoBatches is not implemented in the old physical planner".to_string(),
+        )),
+        LogicalPlan::Limit(LogicalLimit {
+            limit,
+            offset,
+            eager,
+            ..
+        }) => {
             let input_physical = physical_children.pop().expect("requires 1 input");
             let num_partitions = input_physical.clustering_spec().num_partitions();
-            Ok(
-                PhysicalPlan::Limit(Limit::new(input_physical, *limit, *eager, num_partitions))
-                    .arced(),
-            )
+            Ok(PhysicalPlan::Limit(Limit::new(
+                input_physical,
+                *limit,
+                *offset,
+                *eager,
+                num_partitions,
+            ))
+            .arced())
         }
         LogicalPlan::TopN(LogicalTopN {
             sort_by,
             descending,
             nulls_first,
+            offset,
             limit,
             ..
         }) => {
@@ -166,6 +193,7 @@ pub(super) fn translate_single_logical_node(
                 descending.clone(),
                 nulls_first.clone(),
                 *limit,
+                *offset,
                 num_partitions,
             ))
             .arced())
@@ -541,15 +569,13 @@ pub(super) fn translate_single_logical_node(
                 .arced(),
             )
         }
-        LogicalPlan::Intersect(_) => Err(DaftError::InternalError(
-            "Intersect should already be optimized away".to_string(),
-        )),
-        LogicalPlan::Union(_) => Err(DaftError::InternalError(
-            "Union should already be optimized away".to_string(),
-        )),
-        LogicalPlan::SubqueryAlias(_) => Err(DaftError::InternalError(
-            "Alias should already be optimized away".to_string(),
-        )),
+        LogicalPlan::Intersect(_)
+        | LogicalPlan::Union(_)
+        | LogicalPlan::SubqueryAlias(_)
+        | LogicalPlan::Offset(_) => Err(DaftError::InternalError(format!(
+            "Logical plan operator {} should already be optimized away",
+            logical_plan.name()
+        ))),
         LogicalPlan::Window(_window) => Err(DaftError::NotImplemented(
             "Window functions are currently only supported on the native runner.".to_string(),
         )),
