@@ -255,9 +255,20 @@ impl PhysicalPlan {
                     acc_selectivity: input_stats.acc_selectivity * estimated_selectivity,
                 }
             }
-            Self::Limit(Limit { input, limit, .. }) | Self::TopN(TopN { input, limit, .. }) => {
+            Self::Limit(Limit {
+                input,
+                limit,
+                offset,
+                ..
+            })
+            | Self::TopN(TopN {
+                input,
+                limit,
+                offset,
+                ..
+            }) => {
                 let input_stats = input.approximate_stats();
-                let limit = *limit as usize;
+                let limit = (*limit + offset.unwrap_or(0)) as usize;
                 let limit_selectivity = if input_stats.num_rows > limit {
                     if input_stats.num_rows == 0 {
                         0.0
@@ -459,10 +470,10 @@ impl PhysicalPlan {
                     input.clone(), projection.clone(), clustering_spec.clone(),
                 ).unwrap()),
 
-                Self::ActorPoolProject(ActorPoolProject {projection, ..}) => Self::ActorPoolProject(ActorPoolProject::try_new(input.clone(), projection.clone()).unwrap()),
+                Self::ActorPoolProject(ActorPoolProject {projection, udf_properties, ..}) => Self::ActorPoolProject(ActorPoolProject::try_new(input.clone(), projection.clone(), udf_properties.clone()).unwrap()),
                 Self::Filter(Filter { predicate, estimated_selectivity,.. }) => Self::Filter(Filter::new(input.clone(), predicate.clone(), *estimated_selectivity)),
-                Self::Limit(Limit { limit, eager, num_partitions, .. }) => Self::Limit(Limit::new(input.clone(), *limit, *eager, *num_partitions)),
-                Self::TopN(TopN { sort_by, descending, nulls_first, limit, num_partitions, .. }) => Self::TopN(TopN::new(input.clone(), sort_by.clone(), descending.clone(), nulls_first.clone(), *limit, *num_partitions)),
+                Self::Limit(Limit { limit, offset, eager, num_partitions, .. }) => Self::Limit(Limit::new(input.clone(), *limit, *offset, *eager, *num_partitions)),
+                Self::TopN(TopN { sort_by, descending, nulls_first, limit, offset, num_partitions, .. }) => Self::TopN(TopN::new(input.clone(), sort_by.clone(), descending.clone(), nulls_first.clone(), *limit, *offset,*num_partitions)),
                 Self::Explode(Explode { to_explode, .. }) => Self::Explode(Explode::try_new(input.clone(), to_explode.clone()).unwrap()),
                 Self::Unpivot(Unpivot { ids, values, variable_name, value_name, .. }) => Self::Unpivot(Unpivot::new(input.clone(), ids.clone(), values.clone(), variable_name, value_name)),
                 Self::Pivot(Pivot { group_by, pivot_column, value_column, names, .. }) => Self::Pivot(Pivot::new(input.clone(), group_by.clone(), pivot_column.clone(), value_column.clone(), names.clone())),
