@@ -188,6 +188,7 @@ pub mod pylib {
         can_absorb_filter: bool,
         can_absorb_limit: bool,
         can_absorb_select: bool,
+        supports_count_pushdown: bool,
         display_name: String,
     }
 
@@ -232,6 +233,11 @@ pub mod pylib {
             abc.call_method0(py, pyo3::intern!(py, "display_name"))?
                 .extract::<String>(py)
         }
+
+        fn _supports_count_pushdown(abc: &PyObject, py: Python) -> PyResult<bool> {
+            abc.call_method0(py, pyo3::intern!(py, "supports_count_pushdown"))?
+                .extract::<bool>(py)
+        }
     }
 
     #[pymethods]
@@ -245,6 +251,7 @@ pub mod pylib {
             let can_absorb_limit = Self::_can_absorb_limit(&abc, py)?;
             let can_absorb_select = Self::_can_absorb_select(&abc, py)?;
             let display_name = Self::_display_name(&abc, py)?;
+            let supports_count_pushdown = Self::_supports_count_pushdown(&abc, py)?;
 
             Ok(Self {
                 name,
@@ -255,6 +262,7 @@ pub mod pylib {
                 can_absorb_limit,
                 can_absorb_select,
                 display_name,
+                supports_count_pushdown,
             })
         }
     }
@@ -352,6 +360,10 @@ pub mod pylib {
             false
         }
 
+        fn supports_count_pushdown(&self) -> bool {
+            self.supports_count_pushdown
+        }
+
         fn multiline_display(&self) -> Vec<String> {
             let lines = vec![format!("PythonScanOperator: {}", self.display_name)];
             lines
@@ -681,7 +693,7 @@ pub mod pylib {
             Arc::new(FileFormatConfig::Parquet(default::Default::default())),
             Arc::new(schema),
             Arc::new(Default::default()),
-            Pushdowns::new(None, None, columns.map(Arc::new), None, None),
+            Pushdowns::new(None, None, columns.map(Arc::new), None, None, None),
             None,
         );
         Ok(st.estimate_in_memory_size_bytes(None).unwrap())
