@@ -155,6 +155,8 @@ pub enum DataType {
     Python,
 
     Unknown,
+
+    File,
 }
 
 fn format_struct(fields: &[Field]) -> std::result::Result<String, std::fmt::Error> {
@@ -286,6 +288,9 @@ impl DataType {
             Self::Unknown => Err(DaftError::TypeError(format!(
                 "Can not convert {self:?} into arrow type"
             ))),
+            Self::File => Err(DaftError::TypeError(format!(
+                "Can not convert {self:?} into arrow type"
+            ))),
         }
     }
 
@@ -350,6 +355,13 @@ impl DataType {
                     Field::new("indices", List(Box::new(minimal_indices_dtype)))
                 },
             ]),
+            File => Struct(vec![
+                Field::new("discriminant", UInt8),
+                Field::new("data", Binary),
+                Field::new("url", Utf8),
+                #[cfg(feature = "python")]
+                Field::new("io_config", Python),
+            ]),
             _ => {
                 assert!(self.is_physical());
                 self.clone()
@@ -644,6 +656,15 @@ impl DataType {
         }
     }
 
+    #[inline]
+    pub fn is_file(&self) -> bool {
+        match self {
+            Self::File => true,
+            Self::Extension(_, inner, _) => inner.is_file(),
+            _ => false,
+        }
+    }
+
     #[inline]
     pub fn to_floating_representation(&self) -> DaftResult<Self> {
         let data_type = match self {
@@ -725,6 +746,7 @@ impl DataType {
                 | Self::SparseTensor(..)
                 | Self::FixedShapeSparseTensor(..)
                 | Self::Map { .. }
+                | Self::File
         )
     }
 
