@@ -341,6 +341,11 @@ impl PyDataType {
         Ok(DataType::Python.into())
     }
 
+    #[staticmethod]
+    pub fn file() -> PyResult<Self> {
+        Ok(DataType::File.into())
+    }
+
     pub fn to_arrow<'py>(&self, py: Python<'py>) -> PyResult<Bound<'py, PyAny>> {
         let pyarrow = py.import(pyo3::intern!(py, "pyarrow"))?;
         match &self.dtype {
@@ -521,6 +526,10 @@ impl PyDataType {
         self.dtype.is_temporal()
     }
 
+    pub fn is_file(&self) -> bool {
+        self.dtype.is_file()
+    }
+
     pub fn fixed_size(&self) -> PyResult<usize> {
         self.dtype
             .fixed_size()
