@@ -138,6 +138,7 @@ struct ParquetWriter<B: StorageBackend> {
     partition_values: Option<RecordBatch>,
     storage_backend: B,
     file_writer: Option<SerializedFileWriter<B::Writer>>,
+    total_bytes_written: usize,
 }
 
 impl<B: StorageBackend> ParquetWriter<B> {
@@ -159,6 +160,7 @@ impl<B: StorageBackend> ParquetWriter<B> {
             partition_values,
             storage_backend,
             file_writer: None,
+            total_bytes_written: 0,
         }
     }
 
@@ -260,7 +262,6 @@ impl<B: StorageBackend> AsyncFileWriter for ParquetWriter<B> {
         if self.file_writer.is_none() {
             self.create_writer().await?;
         }
-        let starting_bytes_written = self.bytes_written();
         let record_batches = data.get_tables()?;
 
         let row_group_writer_thread_handle = {
@@ -329,9 +330,11 @@ impl<B: StorageBackend> AsyncFileWriter for ParquetWriter<B> {
             .await
             .map_err(|e| DaftError::ParquetError(e.to_string()))??;
 
+        let bytes_written = file_writer.bytes_written() - self.total_bytes_written;
+        self.total_bytes_written = file_writer.bytes_written();
         self.file_writer.replace(file_writer);
 
-        Ok(self.bytes_written() - starting_bytes_written)
+        Ok(bytes_written)
     }
 
     async fn close(&mut self) -> DaftResult<Self::Result> {
@@ -379,13 +382,10 @@ impl<B: StorageBackend> AsyncFileWriter for ParquetWriter<B> {
     }
 
     fn bytes_written(&self) -> usize {
-        match &self.file_writer {
-            None => unreachable!("File writer must be created before bytes_written can be called"),
-            Some(writer) => writer.bytes_written(),
-        }
+        self.total_bytes_written
     }
 
     fn bytes_per_file(&self) -> Vec<usize> {
-        vec![self.bytes_written()]
+        vec![self.total_bytes_written]
     }
 }
