@@ -49,10 +49,18 @@ fn should_enable_chrome_trace() -> bool {
 pub mod pylib {
     use std::sync::LazyLock;
 
+    use common_logging::GLOBAL_LOGGER;
     use common_tracing::{init_opentelemetry_providers, init_tracing};
     use pyo3::prelude::*;
 
-    static LOG_RESET_HANDLE: LazyLock<pyo3_log::ResetHandle> = LazyLock::new(pyo3_log::init);
+    static LOG_RESET_HANDLE: LazyLock<pyo3_log::ResetHandle> = LazyLock::new(|| {
+        let py_logger = Box::new(pyo3_log::Logger::default());
+        let handle = py_logger.reset_handle();
+
+        GLOBAL_LOGGER.set_base_logger(py_logger);
+        log::set_boxed_logger(Box::new(GLOBAL_LOGGER.clone())).unwrap();
+        handle
+    });
 
     #[pyfunction]
     pub fn version() -> &'static str {
@@ -120,6 +128,7 @@ pub mod pylib {
         common_resource_request::register_modules(m)?;
         common_file_formats::python::register_modules(m)?;
         common_scan_info::register_modules(m)?;
+        daft_ai::register_modules(m)?;
         daft_catalog::register_modules(m)?;
         daft_connect::register_modules(m)?;
         daft_context::register_modules(m)?;
@@ -143,6 +152,7 @@ pub mod pylib {
         daft_session::register_modules(m)?;
         daft_sql::register_modules(m)?;
         daft_shuffles::python::register_modules(m)?;
+        daft_file::python::register_modules(m)?;
         // Register testing module
         let testing_module = PyModule::new(m.py(), "testing")?;
         m.add_submodule(&testing_module)?;
@@ -173,11 +183,12 @@ pub mod pylib {
         functions_registry.register::<daft_functions_serde::SerdeFunctions>();
         functions_registry.register::<daft_functions_temporal::TemporalFunctions>();
         functions_registry.register::<daft_functions::HashFunctions>();
-        functions_registry.register::<daft_functions::StructFunctions>();
+        functions_registry.register::<daft_functions::ConversionFunctions>();
         functions_registry.register::<daft_functions::distance::DistanceFunctions>();
         functions_registry.register::<daft_functions_tokenize::TokenizeFunctions>();
 
         functions_registry.add_fn(daft_functions::coalesce::Coalesce);
+        functions_registry.add_fn(daft_file::File);
         functions_registry
             .add_fn(daft_functions::monotonically_increasing_id::MonotonicallyIncreasingId);
         functions_registry.register::<daft_functions::distance::DistanceFunctions>();
