@@ -16,12 +16,6 @@ from daft.io._generator import read_generator
 from daft.recordbatch.recordbatch import RecordBatch
 from tests.conftest import get_tests_daft_runner_name
 
-pytestmark = pytest.mark.skipif(
-    get_tests_daft_runner_name() != "ray"
-    or get_context().daft_execution_config.use_experimental_distributed_engine is True,
-    reason="shuffle tests are not yet supported for flotilla",
-)
-
 
 def generate(num_rows: int, bytes_per_row: int):
     data = {
@@ -180,8 +174,9 @@ def test_pre_shuffle_merge_randomly_sized_partitions(pre_shuffle_merge_ctx, inpu
 
 
 @pytest.mark.skipif(
-    get_tests_daft_runner_name() != "ray",
-    reason="shuffle tests are meant for the ray runner",
+    get_tests_daft_runner_name() != "ray"
+    or get_context().daft_execution_config.use_experimental_distributed_engine is True,
+    reason="shuffle tests are meant for the ray runner and flight shuffle is not yet supported for flotilla",
 )
 @pytest.mark.parametrize(
     "input_partitions, output_partitions",
